/*
 * ============================================================================
 * GNU Lesser General Public License
 * ============================================================================
 *
 * JasperReports - Free Java report-generating library.
 * Copyright (C) 2001-2005 JasperSoft Corporation http://www.jaspersoft.com
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 * 
 * JasperSoft Corporation
 * 185, Berry Street, Suite 6200
 * San Francisco CA 94107
 * http://www.jaspersoft.com
 */

/*
 * Contributors:
 * Peter Severin - peter_p_s@users.sourceforge.net 
 */
package net.sf.jasperreports.compilers;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.Collection;
import java.util.Iterator;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRReport;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JRAbstractJavaCompiler;
import net.sf.jasperreports.engine.design.JRVerifier;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.util.JRProperties;
import net.sf.jasperreports.engine.util.JRSaver;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.groovy.ast.ClassNode;
import org.codehaus.groovy.control.CompilationFailedException;
import org.codehaus.groovy.control.CompilationUnit;
import org.codehaus.groovy.control.CompilerConfiguration;
import org.codehaus.groovy.control.Phases;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;

/**
 * Calculator compiler that uses groovy to compile expressions.
 * 
 * @author Teodor Danciu (teodord@users.sourceforge.net), Peter Severin (peter_p_s@users.sourceforge.net)
 * @version $Id: JRGroovyCompiler.java,v 1.11 2005/07/11 08:09:00 lucianc Exp $
 */
public class JRGroovyCompiler extends JRAbstractJavaCompiler 
{

	/**
	 *  
	 */
	private static final Log log = LogFactory.getLog(JRGroovyCompiler.class);

	public static final String LANGUAGE_GROOVY = "groovy";
	

	/**
	 * @see net.sf.jasperreports.engine.design.JRCompiler#compileReport(net.sf.jasperreports.engine.design.JasperDesign)
	 */
	public JasperReport compileReport(JasperDesign jasperDesign) throws JRException 
	{
		JasperReport jasperReport = null;
	
		if (!LANGUAGE_GROOVY.equals(jasperDesign.getLanguage()))
		{
			throw 
				new JRException(
					"Language \"" + jasperDesign.getLanguage() 
					+ "\" not supported by this report compiler.\n"
					+ "Expecting \"groovy\" instead."
					);
		}
		
		Collection brokenRules = JRVerifier.verifyDesign(jasperDesign);
		if (brokenRules != null && brokenRules.size() > 0) 
		{
			StringBuffer sbuffer = new StringBuffer();
			sbuffer.append("Report design not valid : ");
			int i = 1;
			for (Iterator it = brokenRules.iterator(); it.hasNext(); i++) 
			{
				sbuffer.append("\n\t " + i + ". " + (String) it.next());
			}
			throw new JRException(sbuffer.toString());
		} 
		else 
		{
			//Report design OK
	
			//Generating expressions class source code
			String sourceCode = JRGroovyGenerator.generateClass(jasperDesign);
	
			boolean isKeepJavaFile = JRProperties.getBooleanProperty(JRProperties.COMPILER_KEEP_JAVA_FILE); 
	
			if (isKeepJavaFile) 
			{
				String tempDirStr = JRProperties.getProperty(JRProperties.COMPILER_TEMP_DIR);
	
				File tempDirFile = new File(tempDirStr);
				if (!tempDirFile.exists() || !tempDirFile.isDirectory())
				{
					throw new JRException("Temporary directory not found : " + tempDirStr);
				}
			
				File javaFile = new File(tempDirFile, jasperDesign.getName() + ".groovy");
				
				JRSaver.saveClassSource(sourceCode, javaFile);
			}
			
			String classpath = JRProperties.getProperty(JRProperties.COMPILER_CLASSPATH);
	
			CompilerConfiguration config = new CompilerConfiguration();
			config.setClasspath(classpath);
			CompilationUnit unit = new CompilationUnit(config);
			unit.addSource("calculator", new ByteArrayInputStream(sourceCode.getBytes()));
	
			ClassCollector collector = new ClassCollector();
			unit.setClassgenCallback(collector);
			try 
			{
				unit.compile(Phases.CLASS_GENERATION);
			} 
			catch (CompilationFailedException e) 
			{
				throw new JRException(
					"Errors were encountered when compiling report expressions class file:\n" 
					+ e.toString()
					);
			}
	
			if (collector.classCount == 0) 
			{
				throw new JRException("No groovy class were generated.");
			} 
			else if (collector.classCount > 1) 
			{
				throw new JRException(
					"More than one groovy class were generated.\n"
					+ "Please make sure that you don't use Groovy features such as closures that are not supported by this report compiler.\n"
					);
			}
	
			//Reading class byte codes from compiled class file
			jasperReport = 
				new JasperReport(
					jasperDesign, 
					getClass().getName(),
					collector.bytes
					);
		}
	
		return jasperReport;
	}

	
	/**
	 *
	 */
	private static class ClassCollector extends CompilationUnit.ClassgenCallback 
	{
		public byte[] bytes;
		public int classCount;
	
		/**
		 * @see org.codehaus.groovy.control.CompilationUnit.ClassgenCallback#call(
		 *      org.objectweb.asm.ClassVisitor, 
		 *      org.codehaus.groovy.ast.ClassNode)
		 */
		public void call(ClassVisitor writer, ClassNode node) throws CompilationFailedException 
		{
			classCount++;
			if (bytes == null)
			{
				bytes = ((ClassWriter) writer).toByteArray();
			}
		}
	}


}